import React from 'react';
import Taro from '@tarojs/taro';
import { View } from '@tarojs/components';
import { Button, Notify, Field, Cell, CellGroup } from '@antmjs/vantui';
import LocType from '@/components/LocType';
import IssueType from '@/components/IssueType';
import OrgPicker from '@/components/OrgPicker';
import DatePicker from '@/components/DatePicker';
import Map from '@/components/map';
import Uploader from '@/components/Uploader/index';
import { getIssueStatus } from '@/utils/biz';
import { getDateStr } from '@/utils/date';
import mapIcon from '@/assets/icons/marker.png';
import { getIssueProcess } from '@/services/taissue';

const today = new Date();

export default (props) => {
  const {
    issue,
    readOnly,
    showOrg,
    showExpireDate,
    renderFields,
    renderAction
  } = props;

  const [formData, setFormData] = React.useState({
    typeId: undefined,
    typeName: undefined,
    locId: undefined,
    locName: undefined,
    location: undefined,
    addr: undefined,
    content: undefined,
    attachList: [],
  });
  const [showLocType, setShowLocType] = React.useState(false);
  const [showIssueType, setShowIssueType] = React.useState(false);
  const [showOrgPicker, setShowOrgPicker] = React.useState(false);
  const [showDatePicker, setShowDatePicker] = React.useState(false);
  const [bizStatus, setBizStatus] = React.useState();
  const [issueProcess, setIssueProcess] = React.useState([]);

  const issueReject = React.useMemo(() => {
    if (!issueProcess || !issueProcess.length) return {};

    return issueProcess.filter(x => x.processStatus === 'reject')[0] || {};
  }, [issueProcess]);

  const onLocTypeChange = (_, it) => {
    setFormData({
      ...formData,
      locId: it.typeId,
      locName: it.name,
    });
    setShowLocType(false);
  }

  const onIssueTypeChange = (_, it) => {
    setFormData({
      ...formData,
      typeId: it.typeId,
      typeName: it.name,
    });
    setShowIssueType(false);
  }

  const onOrgChange = (_, it) => {
    setFormData({
      ...formData,
      orgId: it.orgId,
      orgName: it.name,
    });
    setShowOrgPicker(false);
  }

  const onDateChange = (dt) => {
    const date = getDateStr(dt);
    setFormData({
      ...formData,
      expireDate: date,
    });
    setShowDatePicker(false);
  }

  const setFieldChange = (field, value) => {
    setFormData({
      ...formData,
      [field]: value,
    })
  }

  React.useEffect(() => {
    if (issue) {
      setFormData(issue);
      setBizStatus(getIssueStatus(issue));

      if (issue.processStatus == 'reject') {
        getIssueProcess({ pageSize: 100, issueId: issue.issueId }).then(res => {
          setIssueProcess(res.records || []);
        })
      }
    }
  }, [issue]);

  return (
    <View>
      <LocType
        show={showLocType}
        value={formData.locId}
        onCancel={() => setShowLocType(false)}
        onChange={onLocTypeChange}
      />

      <IssueType
        show={showIssueType}
        value={formData.typeName}
        onCancel={() => setShowIssueType(false)}
        onChange={onIssueTypeChange}
      />

      <OrgPicker
        show={showOrgPicker}
        value={formData.orgName}
        onCancel={() => setShowOrgPicker(false)}
        onChange={onOrgChange}
      />

      <DatePicker
        type="date"
        minDate={today}
        show={showDatePicker}
        value={formData.expireDate}
        onCancel={() => setShowDatePicker(false)}
        onChange={onDateChange}
      />

      <Map location={formData.location} />

      <CellGroup>
        <Cell
          title="点位"
          isLink={!readOnly}
          value={formData.locName}
          onClick={() => !readOnly && setShowLocType(true)}
        />
        <Field
          placeholder="请输入地址"
          value={formData.addr}
          leftIcon={mapIcon}
          readonly={readOnly}
          onChange={e => setFieldChange('addr', e.detail)}
        />
      </CellGroup>

      {
        readOnly && bizStatus && (
          <CellGroup style={{ marginTop: '20px' }}>
            <Cell
              title="状态"
              value={bizStatus.label}
            />

            {
              issueReject.processId && (
                <>
                  <Cell title="驳回原因" />
                  <Field
                    readonly
                    type="textarea"
                    autosize={{ minHeight: '120px' }}
                    value={issueReject.processResult}
                  />
                </>
              )
            }
          </CellGroup>
        )
      }


      <CellGroup style={{ marginTop: '20px' }}>
        <Cell
          title="问题分类"
          isLink={!readOnly}
          style={{ marginTop: '20px' }}
          value={formData.typeName}
          onClick={() => !readOnly && setShowIssueType(true)}
        />
        <Cell title="问题描述" border={false} />

        <Field
          type="textarea"
          placeholder="请输入问题描述"
          readonly={readOnly}
          autosize={{ minHeight: '120px' }}
          value={formData.content}
          onChange={e => setFieldChange('content', e.detail)}
        />
      </CellGroup>

      {/* <CellGroup style={{marginTop: '20px'}}>        
        <Cell title="拍照或视频" border={false} />

        <Cell
          renderTitle={
            <Uploader
              value={formData.attachList}
              disabled={readOnly}
              onChange={e => setFieldChange('attachList',e)}
            />
          }
        />
      </CellGroup> */}

      {
        (showOrg || showExpireDate) && (
          <CellGroup style={{ marginTop: '20px' }}>
            {
              showOrg && (
                <Cell
                  title="交办单位"
                  isLink={!readOnly}
                  value={formData.orgName}
                  onClick={() => !readOnly && setShowOrgPicker(true)}
                />
              )
            }
            {
              showExpireDate && (
                <Cell
                  title="办结时间"
                  isLink={!readOnly}
                  value={formData.expireDate}
                  onClick={() => !readOnly && setShowDatePicker(true)}
                />
              )
            }
          </CellGroup>
        )
      }

      {
        renderFields ?
          renderFields(
            formData,
            {
              setFieldChange,
              setFormData,
              showOrgPicker,
              setShowOrgPicker,
            }
          ) : null
      }

      {
        renderAction ? (
          <View style={{ margin: '20px auto' }}>
            {renderAction(formData)}
          </View>
        ) : null
      }
    </View>
  )
}
